----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
-- @date Feb9th-Mar21th, 2019------------------------------------------
-------------------------------------------

lightControl = {}
lightControl.lightKey = "uninitialized";
lightControl.lowBeamKey = "uninitialized";
lightControl.highBeamKey = "uninitialized";
lightControl.fWorkLightsKey = "uninitialized";
lightControl.rWorkLightsKey = "uninitialized";
lightControl.beaconKey = "uninitialized";
lightControl.hazardKey = "uninitialized";
lightControl.signalLKey = "uninitialized";
lightControl.signalRKey = "uninitialized";
lightControl.lightKeyDown = false;
lightControl.lowBeamKeyDown = false;
lightControl.highBeamKeyDown = false;
lightControl.fWorkLightsKeyDown = false;
lightControl.rWorkLightsKeyDown = false;
lightControl.beaconKeyDown = false;
lightControl.hazardKeyDown = false;
lightControl.signalLKeyDown = false;
lightControl.signalRKeyDown = false;
lightControl.soundEnabled = false;
lightControl.toggleSound = createSample("toggleLightSound");
local file = g_currentModDirectory.."switch.ogg";
loadSample(lightControl.toggleSound, file, false);

function lightControl:playSound()
	playSample(lightControl.toggleSound, 1, 0.2, 0, 0, 0);
end;

function lightControl.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function lightControl.registerEventListeners(vehicleType)
	for _, functionName in pairs( { "onPostLoad", "onUpdate" } ) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, lightControl);
	end;
end;

--[[function lightControl:onPreLoad(savegame)
	self.setNextLightsState = Utils.appendedFunction(lightControl.playSound, Lights.setNextLightsState);
	self.setBeaconLightsVisibility = Utils.appendedFunction(lightControl.playSound, Lights.setBeaconLightsVisibility);
end;]]

function lightControl:onPostLoad(savegame)

	if self.spec_lights ~= nil then	
		self.aiNiteLiteUsed = false;
		--lightControl:setLightsTypesMask(self, 0, true, false);
	end;

	lightControl.soundEnabled = true;

end;

function lightControl:onUpdate(dt)

	if g_gui:getIsGuiVisible("Controls") and lightControl.controlsChanged == nil then -- Reload Key Configuration 
		lightControl.controlsChanged = true;
	elseif lightControl.controlsChanged ~= nil and not g_gui:getIsGuiVisible("Controls") then		
		lightControl:setKeys();
		lightControl.controlsChanged = nil;
	end;
	if self.isClient then
		if not self:getIsOperating() and self.isEntered then
			lightControl:overrideLights(self, false);
		elseif self:getIsAIActive() then
			lightControl:overrideLights(self, true);

			if self.spec_lights ~= nil then	
				local spec = self.spec_lights;
				local dayMinutes = g_currentMission.environment.dayTime/(1000*60);
				local needLights = (dayMinutes > (19*60) or dayMinutes < (6*60));
				if needLights and not self.aiNiteLiteUsed then
					if spec.lightsTypesMask <= 0 then -- Will turn on lights only 1 time during the night if they're off.
						local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^0);
						lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
						lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^1);
						lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
						lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^2);
						lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
						self.aiNiteLiteUsed = true;
					end
				elseif not needLights and self.aiNiteLiteUsed then
					lightControl:setLightsTypesMask(self, 0, true, false); -- Will turn off lights only 1 time during the day if they're still on from the night.
					self.aiNiteLiteUsed = false;
				elseif not needLights and spec.lightsTypesMask <= 0 then
					self.aiNiteLiteUsed = false; -- If Daytime and Player already turned off the Lights; the night light procedure is reset.
				end;
			end;

		elseif self.aiNiteLiteUsed and not self:getIsAIActive() then
			self.aiNiteLiteUsed = false; -- If Helper is dimissed; the night light procedure is reset.
		end;
	end;

	if lightControl.soundEnabled then
		lightControl:lightSounds(self);
	end;

	lightControl:stopKeysIfReleased();

end;
	
function lightControl:setToolLights(self)

    if self.spec_attachable ~= nil then
		local spec = self.spec_attachable;
		if spec.attacherVehicle ~= nil then
			local attacherVehicle = spec.attacherVehicle;
			if attacherVehicle.getLightsTypesMask ~= nil then
				spec:setLightsTypesMask(attacherVehicle:getLightsTypesMask(), true, true);
				spec:setBeaconLightsVisibility(attacherVehicle:getBeaconLightsVisibility(), true, true);
				spec:setTurnLightState(attacherVehicle:getTurnLightState(), true, true);
			end;
		end;
	end;

end;

function lightControl:setNextLightsState(self)

	if self.spec_lights ~= nil then	
	
		local spec = self.spec_lights;

		local currentLightState = spec.currentLightState + 1;
		if currentLightState > #spec.lightStates or (spec.currentLightState == 0 and spec.lightsTypesMask > 0) then
			currentLightState = 0;
		end;
		local lightsTypesMask = 0;
		if currentLightState > 0 then
			for _, lightType in pairs(spec.lightStates[currentLightState]) do
				lightsTypesMask = bitOR(lightsTypesMask, 2^lightType);
			end;
		end;

		lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
		spec.currentLightState = currentLightState;	
		lightControl:setToolLights(self);
	end;

end;

function lightControl:overrideLights(self, aiActive)

	if self.spec_lights ~= nil then
	
		local spec = self.spec_lights;
			
		if Input.isKeyPressed(Input[lightControl.lightKey]) then
			if not lightControl.lightKeyDown then
				lightControl:setNextLightsState(self);
			end;
			lightControl.lightKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.beaconKey]) then
			if not lightControl.beaconKeyDown then
				lightControl:setToolLights(self);
				self:setBeaconLightsVisibility(not self:getBeaconLightsVisibility(), true, true)
			end;
			lightControl.beaconKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.lowBeamKey]) then
			if not lightControl.lowBeamKeyDown then
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^0);
				lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
				lightControl:setToolLights(self);
			end;
			lightControl.lowBeamKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.highBeamKey]) then
			if not lightControl.highBeamKeyDown then
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^3);
				lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
				lightControl:setToolLights(self);
			end;
			lightControl.highBeamKeyDown = true;
		end;

		if Input.isKeyPressed(Input[lightControl.fWorkLightsKey]) then
			if not lightControl.fWorkLightsKeyDown then
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^2);
				lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
				lightControl:setToolLights(self);
			end;
			lightControl.fWorkLightsKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.rWorkLightsKey]) then
			if not lightControl.rWorkLightsKeyDown then
				local lightsTypesMask = bitXOR(spec.lightsTypesMask, 2^1);
				lightControl:setLightsTypesMask(self, lightsTypesMask, true, false);
				lightControl:setToolLights(self);
			end;
			lightControl.rWorkLightsKeyDown = true;
		end;
		if aiActive then
			if Input.isKeyPressed(Input[lightControl.hazardKey]) then
				if not lightControl.hazardKeyDown then
					local state = Lights.TURNLIGHT_OFF;
					if spec.turnLightState ~= Lights.TURNLIGHT_HAZARD then
						state = Lights.TURNLIGHT_HAZARD;
					end;
					self:setTurnLightState(state);
					lightControl:setToolLights(self);
				end;
				lightControl.hazardKeyDown = true;
			end;
				
			if Input.isKeyPressed(Input[lightControl.signalLKey]) then
				if not lightControl.signalLKeyDown then
					local state = Lights.TURNLIGHT_OFF;
					if spec.turnLightState ~= Lights.TURNLIGHT_LEFT then
						state = Lights.TURNLIGHT_LEFT;
					end;
					self:setTurnLightState(state);
					lightControl:setToolLights(self);
				end;
				lightControl.signalLKeyDown = true;
			end;

			if Input.isKeyPressed(Input[lightControl.signalRKey]) then
				if not lightControl.signalRKeyDown then
					local state = Lights.TURNLIGHT_OFF;
					if spec.turnLightState ~= Lights.TURNLIGHT_RIGHT then
						state = Lights.TURNLIGHT_RIGHT;
					end;
					self:setTurnLightState(state);
					lightControl:setToolLights(self);
				end;
				lightControl.signalRKeyDown = true;
			end;
		end;					

	end;

end;

function lightControl:stopKeysIfReleased()

		if not Input.isKeyPressed(Input[lightControl.lightKey]) and lightControl.lightKeyDown then
			lightControl.lightKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.beaconKey]) and lightControl.beaconKeyDown then
			lightControl.beaconKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.lowBeamKey]) and lightControl.lowBeamKeyDown then
			lightControl.lowBeamKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.highBeamKey]) and lightControl.highBeamKeyDown then
			lightControl.highBeamKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.fWorkLightsKey]) and lightControl.fWorkLightsKeyDown then
			lightControl.fWorkLightsKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.rWorkLightsKey]) and lightControl.rWorkLightsKeyDown then
			lightControl.rWorkLightsKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.hazardKey]) and lightControl.hazardKeyDown then
			lightControl.hazardKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.signalLKey]) and lightControl.signalLKeyDown then
			lightControl.signalLKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.signalRKey]) and lightControl.signalRKeyDown then
			lightControl.signalRKeyDown = false;
		end;	

end;

function lightControl:setLightsTypesMask(self, lightsTypesMask, force, noEventSend)

	local spec = self.spec_lights;
	if lightsTypesMask ~= spec.lightsTypesMask or force then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(VehicleSetLightEvent:new(self, lightsTypesMask), nil, nil, self);
			else
				g_client:getServerConnection():sendEvent(VehicleSetLightEvent:new(self, lightsTypesMask));
			end;
		end;
		local activeLightSetup = spec.realLights.low;
		local inactiveLightSetup = spec.realLights.high;
		if self:getUseHighProfile() then
			activeLightSetup = spec.realLights.high;
			inactiveLightSetup = spec.realLights.low;
		end;
		for _, light in pairs(inactiveLightSetup.lightTypes) do
			setVisibility(light.node, false);
		end;
		local function getIsLightActive(light)
			local lightActive = false;
			for _, lightType in pairs(light.lightTypes) do
				if bitAND(lightsTypesMask, 2^lightType) ~= 0 then
					lightActive = true;
					break;
				end;
			end;
			if light.enableDirection ~= nil then
				local reverserDirection = 1.0;
				if self.getReverserDirection ~= nil then
					reverserDirection = self:getReverserDirection();
				end;
				lightActive = lightActive and light.enableDirection == reverserDirection;
			end;
			if lightActive then
				for _, excludedLightType in pairs(light.excludedLightTypes) do
					if bitAND(lightsTypesMask, 2^excludedLightType) ~= 0 then
						lightActive = false;
						break;
					end;
				end;
			end;
			return lightActive;
		end;
		for _,light in pairs(activeLightSetup.lightTypes) do
			local isActive = getIsLightActive(light);
			if isActive then
				setVisibility(light.node, true);
			else
				local active = false;
				if light.brakeLight ~= nil then
					if light.brakeLight.isActive then
						active = true;
					end;
				end;
				setVisibility(light.node, active);
			end;
			light.isActive = isActive;
		end;
		for _, light in pairs(spec.defaultLights) do
			local isActive = getIsLightActive(light);
			setVisibility(light.node, isActive);
		end;
		for _, light in pairs(spec.shaderDefaultLights) do
			local isActive = getIsLightActive(light);
			local value = 1;
			if not isActive then
				value = 0;
			end;
			local _,y,z,w = getShaderParameter(light.node, "lightControl");
			setShaderParameter(light.node, "lightControl", math.max(value*light.intensity, 0), y, z, w, false);
			if light.toggleVisibility then
				setVisibility(light.node, isActive);
			end;
		end;
		spec.lightsTypesMask = lightsTypesMask;
		SpecializationUtil.raiseEvent(self, "onLightsTypesMaskChanged", lightsTypesMask);
	end;

	return true;

end;

Lights.setLightsTypesMask = function(self, lightsTypesMask, force, noEventSend)

	if not self:getIsAIActive() then
		lightControl:setLightsTypesMask(self, lightsTypesMask, force, noEventSend);	--Forward this function to the real function
	end;

end;


function lightControl:setKeys()

	local xmlFile = loadXMLFile("inputBinding", lightControlRegister.inputsXML)
	
	local i = 0;
	local iCount = 0;
	while true do
		local objKey = string.format("inputBinding.actionBinding(%d)", i);
		local inputKey = getXMLString(xmlFile, objKey.."#action")
		if inputKey == nil or iCount >= 9 then
			break;
		end;
		if inputKey == "TOGGLE_LIGHTS" then
			lightControl.lightKey = getXMLString(xmlFile, objKey.. ".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_BEACON_LIGHTS" then
			lightControl.beaconKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_TURNLIGHT_LEFT" then
			lightControl.signalLKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_TURNLIGHT_RIGHT" then
			lightControl.signalRKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_TURNLIGHT_HAZARD" then
			lightControl.hazardKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_WORK_LIGHT_BACK" then
			lightControl.rWorkLightsKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_WORK_LIGHT_FRONT" then
			lightControl.fWorkLightsKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_HIGH_BEAM_LIGHT" then
			lightControl.highBeamKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
		if inputKey == "TOGGLE_LIGHT_FRONT" then
			lightControl.lowBeamKey = getXMLString(xmlFile, objKey..".binding#input")
			iCount = iCount + 1;
		end;
        i = i + 1;
    end;
	
	--print("Keys Set");

end;

function lightControl:lightSounds(self)

	if self.spec_lights ~= nil then
		if Input.isKeyPressed(Input[lightControl.lightKey]) then
			if not lightControl.lightKeyDown then
				lightControl:playSound();
			end;
			lightControl.lightKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.beaconKey]) then
			if not lightControl.beaconKeyDown then
				lightControl:playSound();
			end;
			lightControl.beaconKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.lowBeamKey]) then
			if not lightControl.lowBeamKeyDown then
				lightControl:playSound();
			end;
			lightControl.lowBeamKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.highBeamKey]) then
			if not lightControl.highBeamKeyDown then
				lightControl:playSound();
			end;
			lightControl.highBeamKeyDown = true;
		end;

		if Input.isKeyPressed(Input[lightControl.fWorkLightsKey]) then
			if not lightControl.fWorkLightsKeyDown then
				lightControl:playSound();
			end;
			lightControl.fWorkLightsKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.rWorkLightsKey]) then
			if not lightControl.rWorkLightsKeyDown then
				lightControl:playSound();
			end;
			lightControl.rWorkLightsKeyDown = true;
		end;

		if Input.isKeyPressed(Input[lightControl.hazardKey]) then
			if not lightControl.hazardKeyDown then
				lightControl:playSound();
			end;
			lightControl.hazardKeyDown = true;
		end;
				
		if Input.isKeyPressed(Input[lightControl.signalLKey]) then
			if not lightControl.signalLKeyDown then
				lightControl:playSound();
			end;
			lightControl.signalLKeyDown = true;
		end;

		if Input.isKeyPressed(Input[lightControl.signalRKey]) then
			if not lightControl.signalRKeyDown then
				lightControl:playSound();
			end;
			lightControl.signalRKeyDown = true;
		end;
			
		--[[if not Input.isKeyPressed(Input[lightControl.lightKey]) and lightControl.lightKeyDown then
			lightControl.lightKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.beaconKey]) and lightControl.beaconKeyDown then
			lightControl.beaconKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.lowBeamKey]) and lightControl.lowBeamKeyDown then
			lightControl.lowBeamKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.highBeamKey]) and lightControl.highBeamKeyDown then
			lightControl.highBeamKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.fWorkLightsKey]) and lightControl.fWorkLightsKeyDown then
			lightControl.fWorkLightsKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.rWorkLightsKey]) and lightControl.rWorkLightsKeyDown then
			lightControl.rWorkLightsKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.hazardKey]) and lightControl.hazardKeyDown then
			lightControl.hazardKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.signalLKey]) and lightControl.signalLKeyDown then
			lightControl.signalLKeyDown = false;
		end;
		if not Input.isKeyPressed(Input[lightControl.signalRKey]) and lightControl.signalRKeyDown then
			lightControl.signalRKeyDown = false;
		end;]]			

	end;

end;